/********************************************************
 * Data Visualization - 2D Matrix Demo                  *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Rodger (Jieliang) Luo                        *
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Demonstrate how to make a simple 2D Matrix  *
 *          data visualization with different color     *
 *          schemes.                                    *
 *                                                      *
 * Usage:                                               *
 *          Press '1', '2', '3', '4' to apply different *
 *          color schemes.                              *
 *          Press '0' to use gray scale.                *
 ********************************************************/

// Layout variables 
float constantHorizontalMargin = 160;
float constantVerticalMargin = 40;

// width and height for each cell
float cellWidth, cellHeight;

// 2D matrix table related variables
Table table;

int numRows, numColumns;
float[][] dataMatrix;
float maxValue, minValue;

String[] titles;
int[] checkouts;

PImage colorBar;
String[] colorPath = {"bwReBlend129b.jpg", "bwReBlend129d.jpg", "bwReBlend129h.jpg", "Colormap4.jpg"};
boolean colorScheme = false;

void setup(){
  size(1920, 1080); // setup the size of the window
  // In Processing 3, the size() function must be the first line of code inside setup()
  // and needs to use numbers instead of variables. 

  // Make the screen resizable.                     
  surface.setResizable(true);
  
  //Put data into Processing table from csv
  table = loadTable("TopDVDs_month.csv", "header");
  
  // Get row and column information
  numRows = table.getRowCount(); 
  numColumns = table.getColumnCount();
  println("Rows: " + numRows + " Columns: " + numColumns);
  
  // Create a 2D array to receive data from the table 
  dataMatrix = new float[numRows][numColumns-2]; // doesn't need the first two columns 
  titles = new String[numRows];
  checkouts = new int[numRows];
  
  for(int i=0; i<numRows; i++){
    for(int j=0; j<numColumns-2; j++){
      dataMatrix[i][j] = table.getFloat(i,j+2);
     
      //Find the max value of the whole data
      if(dataMatrix[i][j] > maxValue){
        maxValue = dataMatrix[i][j];
      }
    }
    
    // put title and checkout data from the table to the array
    titles[i] = table.getString(i, 0);
    checkouts[i] = table.getInt(i, 1);
  }
  
  // Find the min value of the whole data
  minValue = maxValue;
  for(int i=0; i<dataMatrix.length; i++){
    for(int j=0; j<dataMatrix[0].length; j++){
      if(dataMatrix[i][j] < minValue){
        minValue = dataMatrix[i][j];
      }
    }
  }
  
  println("Max Value is " + maxValue);
  println("Min Value is " + minValue);
  
  //modifying the cell size according to window size
  cellWidth = (width - constantHorizontalMargin*2)/(numColumns-2);
  cellHeight = (height - constantVerticalMargin*2)/numRows;
}

void draw(){
  background(250);
  translate(60, 0);
    
  //Draw 2D Matrix
  for(int i=0; i<dataMatrix.length; i++){
    for(int j=0; j<dataMatrix[0].length; j++){
      stroke(255);
      strokeWeight(1);
      if(!colorScheme){
        float greyScale = map(dataMatrix[i][j], minValue, maxValue, 255, 0);
        fill(greyScale);
      }
      else{
        color c = colorBar.get((int)map(dataMatrix[i][j], minValue, maxValue, 2, colorBar.width-2), colorBar.height/2);
        fill(c);
      }
      rect(constantHorizontalMargin + cellWidth*j, constantVerticalMargin 
           + cellHeight*i, cellWidth, cellHeight);
    }
  }
  
  //Draw horizontal labels
  for(int i=0; i<11; i++){
    int year = 2007;
    textSize(10);
    textAlign(CENTER, TOP);
    fill(0);
    text(year+i, constantHorizontalMargin + cellWidth*(6+12*i), 
         constantVerticalMargin +cellHeight*numRows + 5);
         
    //draw the seperation lines
    if(i != 0){
      stroke(#009EE5);
      line(constantHorizontalMargin + cellWidth*(12*i), constantVerticalMargin +cellHeight*numRows + 3,  
           constantHorizontalMargin + cellWidth*(12*i), constantVerticalMargin +cellHeight*numRows + 20);
    }
  }
   
  //Draw vertical labels
  for(int i=0; i<numRows; i++){
    textAlign(RIGHT, CENTER);
    fill(0);
    textSize(10);
    text(titles[i], constantHorizontalMargin-3, constantVerticalMargin +cellHeight*(0.5+i));
    
     //show total number on the right side
    textAlign(LEFT, CENTER);
    text(checkouts[i], width-constantHorizontalMargin+3,constantVerticalMargin +cellHeight*(0.5+i));
  }
  
  //Draw Title
  textAlign(CENTER, CENTER);
  textSize(16);
  text("TOP 50 DVDs FROM 2006 TO 2017 BY MONTH", width/2, constantVerticalMargin/2);
}

void keyPressed(){
    if(key == '0'){
      colorScheme = false;
    }
    else if(key == '1'){
      colorScheme = true;
      colorBar = loadImage(colorPath[0]);
    }
    else if(key == '2'){
      colorScheme = true;
      colorBar = loadImage(colorPath[1]);
    }
    else if(key == '3'){
      colorScheme = true;
      colorBar = loadImage(colorPath[2]);
    }
    else if(key == '4'){
      colorScheme = true;
      colorBar = loadImage(colorPath[3]);
    }
    
    if(key == 's' || key == 'S'){
      saveFrame();
    }
}